#include "CutPolygonTool.h"

#include <list>
#include <cmath>

#include "Polygon.h"
#include "PolyDraw.h"

CutPolygonTool::~CutPolygonTool()
{
	delete split;
}

bool CutPolygonTool::on_mouse_down(GdkEventButton *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	if(event->button == 1)
	{
		if(!split)
			split = new Line(Vertex(x,-y),Vertex(x,-y));
	}
	else if(event->button == 3)
	{
		split->B(Vertex(x,-y));

		// not with zero length
		if(split->getLength() > 0)
		{
			std::list<Polygon *> splitResult;
			for(std::list<Polygon *>::iterator it = draw->selected_polygons.begin(); it != draw->selected_polygons.end();)
			{
				Clip_result cr = (*it)->split(*split,splitResult);
				if(cr != SubjectPolygon)
				{
					delete *it;
					it = draw->selected_polygons.erase(it);
				}
				else ++it;
			}
			draw->selected_polygons.splice(draw->selected_polygons.begin(),splitResult);
		}

		return true;
	}

	return true;
}

bool CutPolygonTool::on_mouse_up(GdkEventButton *event)
{
	if(event->button == 1)
	{
		if(split)
		{
			delete split;
			split = NULL;
			return true;
		}
	}
	return false;
}

bool CutPolygonTool::on_mouse_move(GdkEventMotion *event)
{
	if(!split) return false;

	int x = (int) event->x;
	int y = (int) event->y;

	// update pos
	split->B(Vertex(x,-y));

	return true;
}

bool CutPolygonTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	if(split) 
	{
		double size = draw->size;

		double a = split->getAngle(Line(Vertex(0,0),Vertex(0,100)));
		int x = (int) (sin(a)*size);
		int y = (int) (-cos(a)*size);
		Line line1(split->A(),split->A()+Vertex(x,y));
		Line line2(split->A(),split->A()-Vertex(x,y));

		cr->set_line_width(2.0/size);
		cr->set_source_rgba(0.3, 0.3, 0.6, 0.8);
		cr->move_to(line1.A().x()/size, -line1.A().y()/size);
		cr->line_to(line1.B().x()/size, -line1.B().y()/size);
		cr->stroke();
		cr->move_to(line2.A().x()/size, -line2.A().y()/size);
		cr->line_to(line2.B().x()/size, -line2.B().y()/size);
		cr->stroke();

		return true;
	}
	return false;
}