#include "PolygonTool.h"

#include "PolyDraw.h"
#include "ringvec.h"
#include "PolygonClipper.h"

PolygonTool::PolygonTool(PolyDraw *pd) : Tool(pd),down(false)
{
	draw->polygons.splice(draw->polygons.end(),draw->selected_polygons);
}

bool PolygonTool::on_mouse_down(GdkEventButton *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	// finalize polygon
	if(event->button == 3)
	{
		finalize();
	}
	// draw polygon...
	else if(event->button == 1)
	{ 
		down = true;
		// if empty add two (so we have a line)
		if(construct.empty()) 
			construct.push_back(Vertex(x,-y));
		construct.push_back(Vertex(x,-y));
	}
	return true;
}

bool PolygonTool::on_mouse_up(GdkEventButton *event)
{
	if(event->button == 1)
		down = false;
	return true;
}

bool PolygonTool::on_mouse_move(GdkEventMotion *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	if(construct.empty()) return false;

	if(!down)
	{
		// re-set last line position
		construct.back().x(x);
		construct.back().y(-y);
	}
	else
	{
		construct.push_back(Vertex(x,-y));
	}
	return true;
}

bool PolygonTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	if(construct.empty()) return false;

	double size = draw->size;

	cr->set_line_width(1.0/size);
	cr->set_source_rgba(0.3, 0.3, 0.6, 0.8);

	for(std::list<Vertex>::const_iterator it = construct.begin(); it != construct.end(); ++it)
	{
		std::list<Vertex>::const_iterator itb = decreaseIt(it,construct);
		cr->move_to(itb->x()/size, -itb->y()/size);
		cr->line_to(it->x()/size, -it->y()/size);
		cr->stroke();
	}
	return true;
}

void PolygonTool::eliminateZeroLengthEdges()
{
	std::list<Vertex>::iterator it = construct.begin();
	++it;
	while(it != construct.end())
	{
		std::list<Vertex>::iterator before = it;
		--before;
		if(*before == *it)
			it = construct.erase(it);
		else
			++it;
	}
}

Polygon* PolygonTool::constructPolygon()
{
	eliminateZeroLengthEdges();

	Polygon *newpoly = new Polygon(construct);
	construct.clear();

	// size negative? turn around...
	if(newpoly->size() < 0)
	{
		Polygon *newnewpoly = new Polygon(-(*newpoly));
		delete newpoly;
		newpoly = newnewpoly;
	}
	// validate
	if(!newpoly->valid())
	{
		delete newpoly;
		return NULL;
	}
	return newpoly;
}

void PolygonTool::subtractPolygonFromDrawArea(Polygon *newpoly)
{
		// first subtract
		std::list<Polygon *> clipResult;
		for(std::list<Polygon *>::iterator pit = draw->polygons.begin(); pit != draw->polygons.end();)
		{
			Clip_result cr = (*pit)->clip(*newpoly,Difference,clipResult);
			// hit it? delete it and add result polygons to list of polygons
			if(cr != SubjectPolygon)
			{
				Polygon *toDelete = *pit;
				pit = draw->polygons.erase(pit);
				delete toDelete;
				if(draw->polygons.empty()) break;
			}
			else ++pit;
		}
		draw->polygons.splice(draw->polygons.end(),clipResult);
}