#include "SelectTool.h"

#include "Polygon.h"
#include "PolyDraw.h"

SelectTool::~SelectTool()
{
}

bool SelectTool::on_mouse_down(GdkEventButton *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	Vertex point(x,-y);

	// select
	if(event->button == 1)
	{
		selectPolygonAtPoint(point);
		down1 = true;
	}
	// deselect
	else if(event->button == 3)
	{
		deselectPolygonAtPoint(point);
		down3 = true;
	}
	return true;
}

bool SelectTool::on_mouse_up(GdkEventButton *event)
{
	if(event->button == 1)
	{
		down1 = false;
	}
	else if(event->button == 3)
	{
		down3 = false;
	}
	return false;
}

bool SelectTool::on_mouse_move(GdkEventMotion *event)
{
	if(!down1 && !down3) return false;

	int x = (int) event->x;
	int y = (int) event->y;

	Vertex point(x,-y);

	if(down1)
	{
		selectPolygonAtPoint(point);
	}
	else if(down3)
	{
		deselectPolygonAtPoint(point);
	}

	return true;
}

bool SelectTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	return true;
}

bool SelectTool::selectPolygonAtPoint(const Vertex& point) const
{
	bool ok = false;
	// find polygon to select
	for(std::list<Polygon *>::iterator it = draw->polygons.begin(); it != draw->polygons.end(); ++it)
	{
		if((*it)->contains(point))
		{
			draw->selected_polygons.push_back(*it);
			draw->polygons.erase(it);
			ok = true;
			break;
		}
	}
	return ok;
}

bool SelectTool::deselectPolygonAtPoint(const Vertex& point) const
{
	bool ok = false;
	// find polygon to deselect
	for(std::list<Polygon *>::iterator it = draw->selected_polygons.begin(); it != draw->selected_polygons.end(); ++it)
	{
		if((*it)->contains(point))
		{
			draw->polygons.push_back(*it);
			draw->selected_polygons.erase(it);
			ok = true;
			break;
		}
	}
	return ok;
}