
#include <gtkmm/main.h>
#include <gtkmm/window.h>
#include <gtkmm/box.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/radioaction.h>
#include <gtkmm/icontheme.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/enums.h>

#include "PolyDraw.h"
#include "LineTool.h"
#include "AddPolygonTool.h"
#include "SubtractPolygonTool.h"
#include "SelectTool.h"
#include "RemovePolygonTool.h"
#include "CutPolygonTool.h"
#include "GluePolygonTool.h"
#include "SelectAllTool.h"
#include "DrawTool.h"

PolyDraw *poly_draw;

void on_selectAll_clicked() { poly_draw->setMode(new SelectAllTool(poly_draw)); }
void on_selectPolys_clicked() { poly_draw->setMode(new SelectTool(poly_draw)); }
void on_lineTest_clicked() { poly_draw->setMode(new LineTool(poly_draw)); }
void on_polygonAdd_clicked() { poly_draw->setMode(new AddPolygonTool(poly_draw)); }
void on_polygonSub_clicked() { poly_draw->setMode(new SubtractPolygonTool(poly_draw)); }
void on_cutPolys_clicked() { poly_draw->setMode(new CutPolygonTool(poly_draw)); }
void on_delPolys_clicked() { poly_draw->setMode(new RemovePolygonTool(poly_draw)); }
void on_gluePolys_clicked() { poly_draw->setMode(new GluePolygonTool(poly_draw)); }
void on_draw_clicked() { poly_draw->setMode(new DrawTool(poly_draw)); }

int main(int argc, char** argv)
{
	Gtk::Main kit(argc, argv);

	// window
	Gtk::Window win;
	win.set_title("Poly test");
	win.set_default_size(600,600);
	win.set_position(Gtk::WIN_POS_CENTER );

	// toolbar actions
	Gtk::RadioButtonGroup radiogroup;
	Glib::RefPtr<Gtk::RadioAction> 
		draw(Gtk::RadioAction::create(radiogroup,"draw","Draw","Draw or erase with a brush")),
		selectPolys(Gtk::RadioAction::create(radiogroup,"selectPolys","Select polygons","Select and deselect polygons")),
		selectAll(Gtk::RadioAction::create(radiogroup,"selectAll","Select/deselect all polygons","Select or deselect all polygons at once")),
		lineTest(Gtk::RadioAction::create(radiogroup,"lineTest","Test line","Test intersection with a line")),
		polygonAdd(Gtk::RadioAction::create(radiogroup,"polygonAdd","Add polygon","Add a polygon to the landscape")),
		polygonSub(Gtk::RadioAction::create(radiogroup,"polygonSub","Subtract polygon","Subtract a polygon from the landscape")),
		cutPolys(Gtk::RadioAction::create(radiogroup,"cutPolys","Cut Polygon(s)","Cut selected polygons along a line")),
		delPolys(Gtk::RadioAction::create(radiogroup,"delPolys","Delete Polygon(s)","Delete selected polygons")),
		gluePolys(Gtk::RadioAction::create(radiogroup,"gluePolys","Glue Polygons","Glue selected polygons together"));

	Gtk::IconTheme::get_default()->append_search_path("resources");

	draw->set_icon_name("draw");
	selectPolys->set_icon_name("select");
	selectAll->set_icon_name("select_all");
	lineTest->set_icon_name("line");
	polygonAdd->set_icon_name("polygon_add");
	polygonSub->set_icon_name("polygon_sub");
	cutPolys->set_icon_name("cut");
	delPolys->set_icon_name("delete");
	gluePolys->set_icon_name("glue");

	selectAll->signal_activate().connect(sigc::ptr_fun(&on_selectAll_clicked));
	selectPolys->signal_activate().connect(sigc::ptr_fun(&on_selectPolys_clicked));
	lineTest->signal_activate().connect(sigc::ptr_fun(&on_lineTest_clicked));
	polygonAdd->signal_activate().connect(sigc::ptr_fun(&on_polygonAdd_clicked));
	polygonSub->signal_activate().connect(sigc::ptr_fun(&on_polygonSub_clicked));
	cutPolys->signal_activate().connect(sigc::ptr_fun(&on_cutPolys_clicked));
	delPolys->signal_activate().connect(sigc::ptr_fun(&on_delPolys_clicked));
	gluePolys->signal_activate().connect(sigc::ptr_fun(&on_gluePolys_clicked));
	draw->signal_activate().connect(sigc::ptr_fun(&on_draw_clicked));

	Glib::RefPtr<Gtk::ActionGroup> group_tools(Gtk::ActionGroup::create("ToolsToolbar"));
	group_tools->add(draw);
	group_tools->add(selectPolys);
	group_tools->add(selectAll);
	group_tools->add(lineTest);
	group_tools->add(polygonAdd);
	group_tools->add(polygonSub);
	group_tools->add(cutPolys);
	group_tools->add(delPolys);
	group_tools->add(gluePolys);

	// toolbar
	Glib::ustring uidesc = 
		"<ui>"
		"<toolbar name=\"mainToolbar\">"
		"	 <toolitem action=\"selectPolys\"/>"
		"	 <toolitem action=\"selectAll\"/>"
		"	 <toolitem action=\"lineTest\"/>"
		"	 <separator/>"
		"	 <toolitem action=\"polygonAdd\"/>"
		"	 <toolitem action=\"polygonSub\"/>"
		"	 <separator/>"
		"	 <toolitem action=\"cutPolys\"/>"
		"	 <toolitem action=\"delPolys\"/>"
		"	 <toolitem action=\"gluePolys\"/>"
		"	 <toolitem action=\"draw\"/>"
		"	 <separator/>"
		"</toolbar>"
		"</ui>";

	Gtk::VBox box;
	Glib::RefPtr<Gtk::UIManager> uimanager(Gtk::UIManager::create());
	uimanager->add_ui_from_string(uidesc);
	uimanager->insert_action_group(group_tools);
	Gtk::Toolbar* toolbar = static_cast<Gtk::Toolbar*>(uimanager->get_widget("/mainToolbar"));
	toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS );
	toolbar->set_icon_size(Gtk::ICON_SIZE_LARGE_TOOLBAR );

	box.pack_start(*toolbar, Gtk::PACK_SHRINK);

	// poly draw area
	poly_draw = new PolyDraw();
	box.pack_start(*poly_draw, Gtk::PACK_EXPAND_WIDGET);

	polygonAdd->set_active(true);

	win.add(box);
	win.show_all();

	Gtk::Main::run(win);

	delete poly_draw;

	return 0;
}
